type PersistenceType = 'memory' | 'localStorage' | 'sessionStorage' | 'none'

type Attributes =
  | 'tagName'
  | 'className'
  | 'text'
  | 'value'
  | 'type'
  | 'href'
  | 'id'
  | 'name'
  | 'placeholder'
  | 'src'
  | 'title'
  | 'alt'
  | 'role'
  | string

type Capture =  'tap' | 'form' | 'swipe' | 'scroll' | 'video' | 'form' | 'page' | 'mouse-movement' | string

type BaseOptions = {
  /**
   * A list of elements to capture events from. Defaults to ['a', 'button', 'form', 'input', 'select', 'textarea', 'label'].
   * @example ['a', 'button', 'form', 'input']
   */
  elements?: string[]

  /**
   * A list of attributes to capture from the event target. Defaults to `['text', 'className', 'value', 'type', 'tagName', 'href', 'src', 'id', 'name', 'placeholder', 'title', 'alt', 'role']`.
   * @example ['tagName', 'className', 'value', 'type']
   */
  attributes?: Array<Attributes>

  /**
   * A list of selectors to ignore to avoid capturing any sensitive data. Defaults to `[]`.
   * @example ['input[type="password"]'] to ignore all password inputs.
   */
  safelist?: Array<string>

  /**
   * A string to set the persistence method. Defaults to `memory`.
   * - `memory`: The events are stored in memory and are lost when the page is refreshed.
   * - `localStorage`: The events are stored in the browser's local storage and are available after the page is refreshed.
   * - `sessionStorage`: The events are stored in the browser's session storage and are available after the page is refreshed.
   * - `none`: The events are not stored anywhere but are available in the callback function.
   */
  persistence?: PersistenceType

  /**
   * A Callback function fires on captured event stored.
   * @param eventData
   * @example
   * ```js
   * onEventCapture: (eventData) => {
   *  console.log(eventData)
   *  // {
   *  //   event: 'click',
   *  //   target: {
   *  //     selector: 'button',
   *  //     attributes: {
   *  //       text: 'Click Me',
   *  //       className: 'btn btn-primary',
   *  //       type: 'button',
   *  //       tagName: 'BUTTON',
   *  //       ...,
   *  //    },
   *  //   }
   *  //   timestamp: 1620920000000,
   *  //   session_id: '123123123',
   *  //   ...,
   *  // }
   *  // ```
   */
  onEventCapture?: (eventData: Record<string, any>) => void

  /**
   * A list of plugins to use. Defaults to `['tap', 'form', 'page']`.
   * @example ['tap', 'form', 'page', 'scroll', 'video', 'mouse-movement']
   */
  capture?: Array<Capture>

  /**
   * Custom session id, you can provide your own visitor id to track the user or use the default one.
   * If you provide your own visitor id, you should make sure it's unique for each user and if you don't
   * provide any visitor id, it will be generated automatically, but it will persist based on the persistence method.
   * @example '123456789'
   */
  sessionId?: string

  /**
   * Custom payload to be sent with each event.
   * @example { 'user_id': '123456789', 'user_name': 'John Doe', 'user_email': ' }
   */
  payload?: Record<string, any>

  /**
   * Specify the maximum number of events to store in the memory, local storage, or session storage and when the limit is reached, the oldest event will be removed.
   */
  maxEvents?: number

  /**
   * Specify if you want to mask the text content of the elements.
   */
  maskTextContent?: boolean

}

type BindResult = {
  /**
   * The name of the event to added in payload.
   */
  name: string
  /**
   * The target element to bind the event listener to.
   */
  target: EventTarget | EventTarget[] | Window | Document | HTMLElement | Element | Node | NodeList | HTMLCollection
  /**
   * The javascript event type.
   */
  event: string

  /**
   * The event handler.
   * @param event
   */
  callback: (event: any) => Record<string, any> | false

  /**
   * The event options.
   */
  options?: boolean | AddEventListenerOptions

  /**
   * The throttling time in milliseconds.
   */
  throttle?: number,

  /**
   * The condition function to check if the event should be captured.
   */
  condition?: (event:any) => boolean
}

interface Plugin {
  /**
   * The name of the plugin.
   */
  key: string

  /**
   * Method to bind the event listener to the elements using the DOMEvent class.
   * @param options The options passed to the core.
   * @return Array of object target, type, handler, options and name
   * @public
   */
  bind: (options: BaseOptions) => BindResult[]

  /**
   * The callback method, called when the plugin is initialized.
   * @param options The options passed to the core.
   */
  onInit?: (options: BaseOptions) => void

  /**
   *  The callback method, called when the plugin is started.
   */
  onStart?: () => void

  /**
   * The callback method, called when the plugin is stopped.
   */
  onStop?: () => void


  /**
   * The callback method, called before the event is captured.
   * @param event The event to be captured.
   * @returns true if the event should be captured, false otherwise.
   */
  onBeforeCapture?: (event: Event) => boolean

  /**
   * The callback method, called when the event is captured.
   * @param eventData The data of the captured event.
   */
  onEventCapture?: (eventData: Record<string, any>) => void


  /**
   * Get the options of the plugin.
   */
  getOptions?: () => Record<string, any>

}

export { Attributes, BaseOptions, BindResult, Capture, PersistenceType, Plugin };
